/*
 * Copyright (c) 1995-2000 Sun Microsystems, Inc. All Rights Reserved.
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *    
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF THE
 * SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
 * PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR ANY DAMAGES
 * SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
 * THIS SOFTWARE OR ITS DERIVATIVES.
*/

package deviceaccess.servlet;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;


/**
 *
 * This servlet perfoms the actual findDriver() and loadDriver()
 * calls on behalf of the DriverLocator service on a webserver
 * where the drivers are assumed to be located all in one place.
 * At the root of the webserver would be a directory by the name
 * 'drivers'. The actual driver bundles shall exist two level below
 * the this root under directories whose names are the same as the
 * the driver's DRIVER_ID.
 * e.g. If SonyVCRDriver has a DRIVER_ID sony.havi.vhs.vh5-hifi.5 :
 * root-->sony.havi.vhs.vh5-hifi.5-->SonyVCRDriver
 *
 * This servlet takes two input paramters as a query string:
 * MODE indicating: findDriver() or loadDriver() call
 * DRIVER_PATH: a String that represents the DRIVER_ID constructed 
 * using device parameters of the registered device - used to check
 * in the document tree if such a directory exists; if exists then, 
 * the driver also exists.
 *
 */
public class FindDriverServlet extends HttpServlet {
    
    private static final String FIND_MODE = "find";
    private static final String LOAD_MODE = "load";
    private static final String TEST_MODE = "test";
    private static final String DRIVER_ROOT = "drivers/";

    public void doGet(HttpServletRequest request,
                 HttpServletResponse response)
        throws ServletException, IOException
    {         
        // MODE for differenciating between find and load driver calls
        String modeParam = request.getParameter("MODE");
        // DRIVER_PATH - driver_id - name of the directory under root.
        String fileParam = request.getParameter("DRIVER_PATH");        

        if (modeParam != null && modeParam.equals(TEST_MODE)) {
            response.setContentType("text/html");
            PrintWriter out = response.getWriter();
            out.println("test ok");
            return;
        }

        if (modeParam == null || fileParam == null) {
            response.setContentType("text/html");
            PrintWriter out = response.getWriter();
            out.println("Query String: incorrect parameters");
            return; 
        }

        String fullPath = new StringBuffer().
            append(request.getRealPath("/")).
            append(DRIVER_ROOT).
            append(fileParam).
            toString();       

        File file = new File(fullPath);
        String[] fileArray = file.list();

        if ((file != null) && file.exists() && file.isDirectory()) {
            if (fileArray.length > 0) {                
                // we know there is only one driver bundle
                String fileName = new StringBuffer().
                    append(file.getAbsolutePath()).
                    append(File.separator).
                    append(fileArray[0]).
                    toString();

                File driverFile = new File(fileName);

                if (modeParam.equals(FIND_MODE)) {
                    response.setContentType("text/html");
                    PrintWriter out = response.getWriter();
                    findDriver(driverFile, out,
                                 request);            
                }
                else if (modeParam.equals(LOAD_MODE)) {
                    ServletOutputStream out = 
                        response.getOutputStream();            
                    loadDriver(fileName, driverFile, out);            
                }               
            }
        }        
    }
    

    /**
     * Writes "true" if the driver was found else
     * writes "false" to the output stream
     */
    static void findDriver(File driverFile, 
                             PrintWriter out,
                             HttpServletRequest req) {
        try {            
            if (driverFile.isFile()) {
                out.println("true");                
            }                   
        } catch (Exception e) {
            out.println("false");
        } finally {
            if (out != null) out.close();
        }
    }

    /**
     * load the driver bundle 
     * Write out the driver bundle raw data to the output stream
     */
    static void loadDriver(String fileName, 
                             File driverFile, 
                             ServletOutputStream out)
        throws FileNotFoundException, IOException
    {
        FileInputStream fis = null;
        try {            
            if (driverFile.isFile()) {
                fis = new FileInputStream(fileName);
                byte[] buf = new byte[8192]; 
                int bytesRead;
                while ((bytesRead = fis.read(buf, 0, buf.length)) > 0) {
                    out.write(buf, 0, bytesRead);
                }
            }
        }
        catch (IOException e) {
        }         
        finally {            
            try {
                if (out != null) out.close();
                if (fis != null) fis.close();
            } catch (IOException e) {
                
            }
        }   
          
    }    
    
}
