/*
 * @(#)HttpsUser.java	1.2 00/08/03
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun. 
 */

package httpsurl;

import org.osgi.framework.*;

import java.io.*;
import java.net.*;

/*
 * This example service demonstrates how to use capabilities provided
 * by the Java Embedded Server SSL service to establish a connection
 * to an HTTPS server. This is done using the java.net.URL class, so
 * that most details are handled transparently and the SSL service's
 * lower-level API is not invoked. This service uses the following
 * properties:
 *  "com.sun.jes.httpsuser.remotePort" = <remotePort> (default 443)
 *  "com.sun.jes.httpsuser.remoteHost" = <remoteHost> (default "www.sun.com")
 * In addition, if the connection must go through a firewall or proxy
 * host, the SSL service needs the following properties to be set:
 *  "https.proxyHost" = <secureProxyHostName>
 *  "https.proxyPort" = <secureProxyPort>
 *
 */

class HttpsUser implements ServiceListener
{
    final String HOSTPORT_PROPERTY = "com.sun.jes.httpsuser.remotePort";
    final String HOSTNAME_PROPERTY = "com.sun.jes.httpsuser.remoteHost";
    final String SSL_SERVICE = "com.sun.jes.service.ssl.SSLService";

    private String remoteHost;
    private int remotePort;
    private BundleContext bc;

    HttpsUser( BundleContext bc )
    {
        this.bc = bc;
        
        // Look for properties that tell us where to connect.
        remoteHost = System.getProperty( HOSTNAME_PROPERTY );
        if( remoteHost == null )
            remoteHost = "www.sun.com";
        remotePort = Integer.getInteger( HOSTPORT_PROPERTY,
                                             443 ).intValue();

        // Register to listen for whenever the SSL service changes.
        String filter = "(objectClass=" + this.SSL_SERVICE + ")" ;
        try {
            this.bc.addServiceListener( this, filter );
        } catch( InvalidSyntaxException e ) {
        }
    }

    /**
     * The SSL service is running, so we infer that there is an HTTPS
     * protocol handler present. Try to read from the designated HTTPS
     * server.
     * @param hostName Name of HTTPS server.
     * @param port Port number on HTTPS server.
     */
    private void makeURL( String hostName, int port )
    {
        try {
            String urlString = "https://" + hostName + ":" + port;
            URL testURL = new URL( urlString );
            BufferedReader in = new BufferedReader(
                                    new InputStreamReader(
                                        testURL.openStream() ) );
            String line;
            while ( (line = in.readLine() ) != null )
                System.out.println( line );
            in.close();
        } catch ( Exception e ) {
            System.out.println( "Exception using URL" );
            e.printStackTrace();
            return;
        }
    }

    public void cleanUp() 
    {
        this.bc.removeServiceListener( this );
    }


    private ServiceReference sslSR = null;
    /**
     * Start looking for the SSL service. Once it is present (or if it
     * already is, immediately) attempt to establish an HTTPS connection.
     * Since we use only a string to ask for the SSL service, and never
     * cast the result to anything, we have no direct dependence on the
     * SSL service being present, but rather only on a side effect of its
     * being started up.
     */
    public void establishConnection()
    {
        if ( sslSR == null && 
             ( sslSR = this.bc.getServiceReference( SSL_SERVICE ) )
                != null ) {
                makeURL( this.remoteHost, this.remotePort );
        }
        return;
    }

    /**
     * If the SSL Service is registered, we know we have a protocol
     * handler for HTTPS.
     * @param event The event indicating the ServiceChange
     */
    public void serviceChanged( ServiceEvent event )
    {
        // Synchronize on this so we don't receive a multiplicity
        // of notifications simultaneously.
        synchronized( this ) {
            switch( event.getType() )
            {
                case ServiceEvent.UNREGISTERING:
                    sslSR = null;
                    break;
                case ServiceEvent.REGISTERED:
                    establishConnection();
                    break;
            }
        }
        return;
    }

}


