/*
 * Copyright (c) 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * This example demonstrates how to write a bundle which uses the
 * JSSE API (rather than the Java Embedded Server SSL Service, which is
 * not intended for general use) to establish an SSL socket connection
 * to another host. This service uses the following properties:
 *
 * "com.sun.jes.jsseapi.hostName" = <remoteHost> (no default)
 * "com.sun.jes.jsseapi.hostPort" = <remotePort> (default 9090)
 * "javax.net.ssl.trustStore = <path to the "clientcerts.cert" file>
 *
 * The trustStore property, if set, specifies a certificate file to
 * use when negotiating connections with the remote server. If we
 * are connecting to a Java Embedded Server server example, we should
 * use the "clientcerts.cert" file provided:
 *
 * <jes_install_dir>/docs/examples/jsseapiclient/clientcerts.cert
 *
 * This example uses classes exported by the Java Embedded Server SSL
 * service bundle, which must be at least in a RESOLVED state for this
 * service to function. Note that the SSL service must have been started
 * after the trustStore property noted above has been set, or the
 * connection attempt will fail with an "untrusted server cert chain"
 * exception.
 */

package jsseapiclient;

import java.net.*;
import java.io.*; 
import javax.net.ssl.*;
import org.osgi.framework.*;

public class JsseApiClient
{
    // Properties and default values needed for the connection
    final static String HOSTNAME_PROPERTY = "com.sun.jes.jsseapi.hostName" ;
    final static String HOSTPORT_PROPERTY = "com.sun.jes.jsseapi.hostPort" ;
    final static int HOSTPORT_DEFAULT = 9090;
    
    private SSLSocketFactory factory = null;
    private int port = 0;
    private String hostName = null;

    /**
     * Constructor. Look for hostname to which to connect, and create
     * an SSL socket factory.
     */
    public JsseApiClient()
    {
        hostName = System.getProperty( HOSTNAME_PROPERTY );
        if( hostName == null ) {
            throw new IllegalArgumentException( "No hostname to which to connect" );
        }
        port = Integer.getInteger( HOSTPORT_PROPERTY,
                                   HOSTPORT_DEFAULT ).intValue();
        factory = (SSLSocketFactory)SSLSocketFactory.getDefault();
    }

    /**
     * Connect to our specified host and port. Send a GET request and
     * then look for and echo out the response.
     */
    public void connect()
    {
        SSLSocket socket;

        if( factory == null ) {
            throw new NullPointerException( "No socket factory available." );
        }
        try {
            System.out.println( "connecting to " + hostName + ":" + port );
            socket = (SSLSocket)factory.createSocket( hostName, port );
        } catch ( IOException e ) {
            System.out.println( "IOException creating socket" );
            //e.printStackTrace();
            return;
        }
        
        // Send a GET request
        System.out.println( "sslclient service, connecting to host " +
                            hostName + " at port " + port );
        try {
            String msg = "GET https://" + hostName + " HTTP/1.1\n\r\n\r";
            DataOutputStream out = new DataOutputStream( socket.getOutputStream() );

            out.writeBytes( msg );
            out.flush();
        } catch ( IOException e ) {
            //e.printStackTrace();
            return;
        }
        
        try {
            // Read the response
            BufferedReader in = new BufferedReader(
                                    new InputStreamReader(
                                    socket.getInputStream() ) );
            String inputLine;
            while ( ( inputLine = in.readLine() ) != null ) {
                System.out.println( inputLine );
            }
            in.close();
        } catch ( IOException e ) {
            System.out.println( "IOException receiving reply" );
            //e.printStackTrace();
            return;
        }
        return;
    }
}
