/*
 * Copyright (c) 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information").  You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 * This example demonstrates how to write a bundle which uses the
 * JSSE API (rather than the Java Embedded Server SSL Service, which is
 * not intended for general use) to establish an SSL socket and wait
 * for connections. This service uses the following properties:
 *
 * "com.sun.jes.jsseapi.hostPort" = <port> (default 9090)
 * "com.sun.jes.jsseapi.keyStore" = KeyStore file URL (no default)
 *
 * The keyStore URL should be set to point to the "tlscerts.cert" file
 * in this directory:
 * file:<jes_install_dir>/docs/examples/jsseapiserver/tlscerts.cert
 *
 * This example uses classes exported by the Java Embedded Server SSL
 * service bundle, which must be at least in a RESOLVED state for this
 * service to function.
 */

package jsseapiserver;

import com.sun.net.ssl.*;
import java.security.KeyStore;
import javax.net.*;

import java.net.*;
import java.io.*; 
import javax.net.ssl.*;
import org.osgi.framework.*;

public class JsseApiServer
{
    // Default values and properties
    static final String HOSTPORT_PROPERTY = "com.sun.jes.jsseapi.hostPort" ;
    static final String KEYSTORE_PROPERTY = "com.sun.jes.jsseapi.keyStore";
    final static int HOSTPORT_DEFAULT = 9090;

    // Private variables for establishing a connection
    private SSLServerSocketFactory ssf = null;
    private int port = 0;
    private ServerSocket ss = null;
    private Thread listenerThread = null;

    /**
     * Constructor; create our SSL environment. Establish a KeyStore,
     * a KeyManagerFactory, and a ServerSocketFactory.
     * @exception Can throw a number of exceptions based on difficulties
     *            in setting up SSL context.
     */
    public JsseApiServer() throws Exception
    {
        // Look for the port to which we'll connect
        port = Integer.getInteger( HOSTPORT_PROPERTY,
                                   HOSTPORT_DEFAULT ).intValue();

        // Initialize our factory for SSL sockets
        try {
            SSLContext ctx;
            KeyManagerFactory kmf;
            KeyStore ks;
            char[] passphrase = "passphrase".toCharArray();

            ctx = SSLContext.getInstance( "TLS" );
            kmf = KeyManagerFactory.getInstance( "SunX509" );
            ks = KeyStore.getInstance( "JKS" );

            // Find the keystore file and use it to initialize the KeyStore.
            String urlPath = System.getProperty( KEYSTORE_PROPERTY );
            if( urlPath == null ) {
                System.out.println( "no KeyStore property!" );
                throw new IllegalArgumentException( "No KeyStore property" );
            } else {
                if( ! urlPath.startsWith( "file:" ) ) {
                    urlPath = "file:" + urlPath;
                }
            }
                
            InputStream is = null;
            System.out.println( "trying to load keystore " + urlPath );
            try {
                URL u = new URL( urlPath );
                is = u.openStream();
                if( is == null ) {
                    throw new NullPointerException( "Null InputStream from KeyStore" );
                }
            } catch( IOException e ) {
                System.out.println( "Can't load KeyStore file " + urlPath );
                throw e;
            }

            // Load the KeyStore and initialize the KeyManagerFactory
            ks.load( is, passphrase );
            kmf.init( ks, passphrase );
            ctx.init( kmf.getKeyManagers(), null, null );

            ssf = ctx.getServerSocketFactory();
        }
        catch( Exception e ) {
            System.out.println( "Can't init SSL factory" );
            e.printStackTrace();
            throw e;
        }
        return;
    }

    /**
     * Create a server socket on the local host and wait on it for
     * someone to connect. After one connection, clean up.
     * @exception UknownHostException If there is a problem getting the
     *                                InetAddress of the localhost.
     * @exception IOException If there is a problem creating the ServerSocket.
     */
    public void waitForConnection() 
    {
        InetAddress myAddr = null;

        try {
            myAddr = InetAddress.getLocalHost();
            ss = ssf.createServerSocket( port, 0, myAddr );
        }
        catch ( UnknownHostException e ) {
            return;
        }
        catch ( IOException e ) {
            e.printStackTrace();
            return;
        }
        SocketListener l = new SocketListener( ss, myAddr );
        listenerThread = new Thread( l );
        listenerThread.start();
    }

    /**
     * Shutdown. If we're still waiting for a connection, this will
     * bounce us out of our accept() call on the server socket.
     */
    public void shutdown()
    {
        if( ss != null ) {
            try {
                ss.close();
            }
            catch( IOException e ) {
                // Eat it
            }
            ss = null;
        }
        try {
            listenerThread.join();
        }
        catch( InterruptedException e ) {
            e.printStackTrace();
        }
        return;
    }
}
