/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved. 
 * 
 * This software is the confidential and proprietary information of Sun 
 * Microsystems, Inc. ("Confidential Information"). You shall not 
 * disclose such Confidential Information and shall use it only in 
 * accordance with the terms of the license agreement you entered into 
 * with Sun. 
 *
 * 
 * Log2Http
 *
 * This example service listens to Log service messages and forwards them
 * to a server via the HTTP mechanism.  Both the client and the server run
 * on the same host.  If another host is desired for the server, the name
 * of that server must be specified in "hostname" and a different Socket()
 * statement must be used.
 *
 * Messages are buffered until a "threshold" count is reached.  Then a
 * set of messages is forwarded to the HTTP server.
 */

package log2http;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Enumeration;
import java.util.Vector;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogEntry;
import org.osgi.service.log.LogListener;
import org.osgi.service.log.LogService;
import org.osgi.service.log.LogReaderService;

public class Log2Http implements LogListener {

    public final static String LOG_CLASS = "org.osgi.service.log.LogService";
    public final static String LOG_READER_CLASS = "org.osgi.service.log.LogReaderService";
    private final static int MESSAGES_MAX = 10;
    private final static int MESSAGES_THRESHOLD = MESSAGES_MAX / 2;
    //private final static String METHOD_GET  = "GET";
    //private final static String METHOD_HEAD = "HEAD";
    private final static String METHOD_POST = "POST";
    private final static String HTTP_VERSION = "HTTP/1.0";

    private BufferedReader fromServer;            // Stream used to get responses from server
    private PrintWriter toServer;                 // Stream used to send requests to server

    //String hostname = "your.server.name.here";  // The name of the server if using "hostname"
    InetAddress hostaddress = null;               // The InetAddress of the server
    int port = 9092;                              // The port of the server


    /* Create a Log and a LogReader.  Listen for messages, buffer them and then pass
     * groups of message over to the HTTP server.
     */
    Log2Http( BundleContext bc ) {
        try {
  
            // Use the bundle context to create the Log and LogReader services.
            getServices( bc );
            if (log == null) {
                System.err.println( "Unable to get the Log service reference!" );
                return;
            }
            if (logreader == null) {
                System.err.println( "Unable to get the LogReader service reference!" );
                return;
            }

            // Pick up the InetAddress of our host
            // (Comment-out the line below if you use "hostname" from above)
            hostaddress = InetAddress.getLocalHost();

            // Set up the listener for the messages to be logged.
            logreader.addLogListener( this );

            // Generate some fake Log messages
            log.log(LogService.LOG_ERROR,   "Log error 1");
            log.log(LogService.LOG_WARNING, "Log warning 1");
            log.log(LogService.LOG_INFO,    "Log info 1");
            log.log(LogService.LOG_DEBUG,   "Log msg 1");

            log.log(LogService.LOG_ERROR,   "Log error 2");
            log.log(LogService.LOG_WARNING, "Log warning 2");
            log.log(LogService.LOG_INFO,    "Log info 2");
            log.log(LogService.LOG_DEBUG,   "Log msg 2");

            log.log(LogService.LOG_ERROR,   "Log error 3");
            log.log(LogService.LOG_WARNING, "Log warning 3");
            log.log(LogService.LOG_INFO,    "Log info 3");
            log.log(LogService.LOG_DEBUG,   "Log msg 3");

            /*
             * Send any remaining messages.  In the meantime, other
             * bundles could be logging messages.
             */
            while (messages.size() > 0) {
                sendMessages(); 
            }

            // Remove the LogListener
            logreader.removeLogListener( this );

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /* Get references to the Log and LogReader services.  Synchronize things just
     * in case the Log and LogReader services are being eliminated at this time.
     */
    private LogService log = null;                // Reference to the Log service object
    private LogReaderService logreader = null;    // Reference to the LogReader service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
            if ((sr = bc.getServiceReference( LOG_READER_CLASS )) != null) {
                logreader = (LogReaderService) bc.getService( sr );
            }
        }
    }

    /*
     * LogListener implementation.
     *
     * Intercept Log messages here as they are logged.  Buffer a bunch of them
     * and send them out over HTTP to the remote HTTP server when enough messages
     * have been accumulated.
     */
    private Vector messages = null;               // Storage for all messages
    public void logged( LogEntry entry ) {
        try {
            String message = entry.getMessage();

            if (messages == null) {
                messages = new Vector( MESSAGES_MAX, MESSAGES_MAX );
            }

            System.out.println( "Store #" + messages.size() + ":" + message );
            messages.addElement( message );
            if (messages.size() >= MESSAGES_THRESHOLD) {
                sendMessages();
            }

        } catch (Exception e) {
            /* Ignore exceptions */
        }
    }

    /*
     * Send some messages via HTTP to a remote server.
     *
     */
    private void sendMessages() {
        int count = MESSAGES_THRESHOLD;  // Assume max number of messages to send
        String message;

        try {

            // If the buffer isn't completely full, count only what's left
            if (messages.size() < MESSAGES_THRESHOLD) {
                count = messages.size();
            }

            // Connect to the server, which is the same machine hosting the client
            // Socket s = new Socket( hostname, port );
            Socket s = new Socket( hostaddress, port );

            // Get the I/O streams used to communicate with the server
            InputStream sin = s.getInputStream();
            fromServer = new BufferedReader(new InputStreamReader( sin ));
            OutputStream sout = s.getOutputStream();
            toServer = new PrintWriter(new OutputStreamWriter( sout ));

            // Send all of the messages in this request from the FIFO buffer.
            while (count-- > 0) {
                message = (String) messages.elementAt(0);
                System.out.println( "Sending: " + message );
                toServer.print( METHOD_POST + " ;\"" + message + "\" " + HTTP_VERSION + "\n" );
                messages.removeElementAt(0);
            }
            toServer.print( "\n" );  // Terminator for the request
            toServer.flush();

            // Receive the server's response (Assume it's text)
            System.out.println( "----------------------------------------" );
            for ( String l = null; (l = fromServer.readLine()) != null; )
                System.out.println(l);
            System.out.println( "----------------------------------------\n" );

            // Shut down the streams
            toServer.close();
            fromServer.close();
            s.close();

        } catch (Exception e) {
            System.err.println( "Error: " + e.getMessage() );
            messages.removeAllElements();
        }
    }

}
