/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * LogEnumeration
 *
 * Log messages using all log APIs and all log levels.  Then enumerate
 * through the set of messages and filter only those generated by this
 * example.
 *
 */

package logenumeration;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogEntry;
import org.osgi.service.log.LogService;
import org.osgi.service.log.LogReaderService;
import java.util.Date;
import java.util.Enumeration;

public class LogEnumeration {

    private static final String API_PREFIX = "API Test ";

    LogEnumeration(BundleContext bc) {
        try {

            // Use the bundle context to create the Log service reference.
            getServices( bc );
            if (log == null ) {
                System.err.println( "Unable to get Log service reference!" );
                return;
            }
            if (logreader == null ) {
                System.err.println( "Unable to get LogReader service reference!" );
                return;
            }

            // API test #1
            System.out.println( "public void log(int level, String msg)" );
            log.log( LogService.LOG_ERROR,   API_PREFIX + "1: Log error" );
            log.log( LogService.LOG_WARNING, API_PREFIX + "1: Log warning" );
            log.log( LogService.LOG_INFO,    API_PREFIX + "1: Log info" );
            log.log( LogService.LOG_DEBUG,   API_PREFIX + "1: Log debug" );

            // API test #2
            System.out.println( "public void log(int level, String msg, Throwable e)" );
            log.log( LogService.LOG_ERROR,   API_PREFIX + "2: Log error", new IllegalArgumentException("error") );
            log.log( LogService.LOG_WARNING, API_PREFIX + "2: Log warning", new IllegalArgumentException("warning") );
            log.log( LogService.LOG_INFO,    API_PREFIX + "2: Log info", new IllegalArgumentException("info") );
            log.log( LogService.LOG_DEBUG,   API_PREFIX + "2: Log debug", new IllegalArgumentException("debug") );

            // API test #3
            System.out.println( "public void log(ServiceReference sr, int level, String msg)" );
            log.log( sr, LogService.LOG_ERROR,   API_PREFIX + "3: Log error" );
            log.log( sr, LogService.LOG_WARNING, API_PREFIX + "3: Log warning" );
            log.log( sr, LogService.LOG_INFO,    API_PREFIX + "3: Log info" );
            log.log( sr, LogService.LOG_DEBUG,   API_PREFIX + "3: Log debug" );

            // API test #4
            System.out.println( "public void log(ServiceReference sr, int level, String msg, Throwable e)" );
            log.log( sr, LogService.LOG_ERROR,   API_PREFIX + "4: Log error", new IllegalArgumentException("error") );
            log.log( sr, LogService.LOG_WARNING, API_PREFIX + "4: Log warning", new IllegalArgumentException("warning") );
            log.log( sr, LogService.LOG_INFO,    API_PREFIX + "4: Log info", new IllegalArgumentException("info") );
            log.log( sr, LogService.LOG_DEBUG,   API_PREFIX + "4: Log debug", new IllegalArgumentException("debug") );

            System.out.println();

            /* Enumerate over the items currently in the log and show each
             * entry's contents.
             */
            for( Enumeration e = logreader.getLog(); e.hasMoreElements(); ) {
                showEntry ( (LogEntry) e.nextElement() );
            }

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /*
     * Get references to the Log and LogReader services.
     */
    private final static String LOG_CLASS = "org.osgi.service.log.LogService";
    private final static String LOG_READER_CLASS = "org.osgi.service.log.LogReaderService";
    private LogService log = null;                // Reference to the Log service object
    private LogReaderService logreader = null;    // Reference to the LogReader service object
    ServiceReference sr = null;
    synchronized void getServices( BundleContext bc ) {
        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
            if ((sr = bc.getServiceReference( LOG_READER_CLASS )) != null) {
                logreader = (LogReaderService) bc.getService( sr );
            }
        }
    }

    /* Show the contents of a Log entry.
     *
     * Returns true if there's an error in tne entry.
     */
    private boolean showEntry( LogEntry entry ) {
        Bundle b = entry.getBundle();
        int level = (int) entry.getLevel();
        long time = entry.getTime();
        String message = entry.getMessage();
        ServiceReference sr = entry.getServiceReference();
        Throwable e = entry.getException();

        // Show only those entries that were logged by this bundle
        if (message.startsWith( API_PREFIX ) == true) {

            // Don't show the bundle stuff (too long)
            // System.out.println( "   Bundle: " + b );

            // Show the severity level information
            System.out.print( "           Level " + level + ": " );
            switch ( level ) {
                case LogService.LOG_ERROR:
                    System.out.print( "ERROR  " );
                    break;
                case LogService.LOG_WARNING:
                    System.out.print( "WARNING" );
                    break;
                case LogService.LOG_INFO:
                    System.out.print( "INFO   " );
                    break;
                case LogService.LOG_DEBUG:
                    System.out.print( "DEBUG  " );
                    break;
                default:
                    System.out.print( "  ???  " );
                    return true;
            }

            // Show the time that the message was logged
            System.out.println( "  at  " + new Date(time).toString() );

            // Show the message that was logged
            System.out.println( "           Message: " + message );

            /*
             * Show the service reference information (i.e., the service
             * that logged the message) and any exception messages.
             */
            System.out.print( "Service, Exception: " + sr );
            if (e == null) {
                System.out.println( ", None." );
            } else {
                System.out.println( ", \"" + e.getMessage() + "\"" );
            }

            System.out.println();

        } // if

        return false;
    }

}
