/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved. 
 * 
 * This software is the confidential and proprietary information of Sun 
 * Microsystems, Inc. ("Confidential Information"). You shall not 
 * disclose such Confidential Information and shall use it only in 
 * accordance with the terms of the license agreement you entered into 
 * with Sun. 
 *
 * 
 * LogExample
 *
 * This example obtains a reference to the log service and then logs
 * some messages.
 *
 * The example produces no output.  However, another bundle can register
 * a LogListener to be notified when the log receives messages.  This
 * example first calls the getServices() method to obtain the log service
 * reference.  This functionality is placed in its own synchronized method
 * to make sure that another bundle is not uninstalling the log service
 * while this bundle is attempting to obtain it.  You can also expand
 * getServices() if you also want to obtain other services.
 *
 * The example does a quick check to make sure that getServices() actually
 * obtains the log service reference.  If it doesn't, this bundle must exit.
 *
 * The example logs four types of messages, each with its own unique message.
 * Finally, the log service reference is set to null to avoid memory leaks.
 * The log service may no longer be needed, so the reference can be made null
 * to reclaim memory.  If you call getServices() at this point, it will
 * obtain the reference again so more messages can be logged.
 *
 */

package logexample;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogService;

public class LogExample {

    LogExample( BundleContext bc ) {
        try {

            // Use the bundle context to create the Log service reference.
            getServices( bc );
            if (log == null) {
                System.err.println( "Unable to get the Log service reference!" );
                return;
            }

            // Generate some fake Log messages.
            log.log(LogService.LOG_ERROR,   "Log error");
            log.log(LogService.LOG_WARNING, "Log warning");
            log.log(LogService.LOG_INFO,    "Log info");
            log.log(LogService.LOG_DEBUG,   "Log msg");

            /*
             * Avoid memory leaks.  Even though this is at the end of this
             * example, it's good to do this in case this example is used
             * later and expanded to perform other tasks.
             */
            log = null;

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /* Get the Log service reference.  This is synchronized, just in case
     * there are attempts to uninstall the Log service itself from the
     * framework while this bundle is trying to get the reference.
     */
    public final static String LOG_CLASS = "org.osgi.service.log.LogService";
    private LogService log = null;  // Reference to the Log service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
        }
    }

}
