/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved. 
 * 
 * This software is the confidential and proprietary information of Sun 
 * Microsystems, Inc. ("Confidential Information"). You shall not 
 * disclose such Confidential Information and shall use it only in 
 * accordance with the terms of the license agreement you entered into 
 * with Sun. 
 *
 * 
 * LogFindMaxEntries
 *
 * Start with an initial, arbitrarily chosen minimum and maximum value.
 * Select the mid-point between the minimum and maximum value and use
 * that as the log size to be tried.  If that size works, record it as
 * the new minimum size; otherwise, record the value that caused the
 * error as the new maximum size.  Eventually, the difference between
 * the minimum and maximum sizes is less than or equal to 1.  At that
 * point, the minimum value is the maximum number of entries that can
 * be used for the log.
 * 
 */

package logfindmaxentries;

import org.osgi.framework.BundleContext;
import org.osgi.framework.Configurable;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogService;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

public class LogFindMaxEntries {

    Class logClass = null;  // The LogConfigurationObject reference

    LogFindMaxEntries( BundleContext bc ) {
        try {

            // Use the bundle context to create the Log service reference.
            getServices( bc );
            if (log == null) {
                System.err.println( "Unable to get Log service reference!" );
                return;
            }


            if (log instanceof Configurable) {

                /*
                 * Get the logConfigurationObject and its type (class).  Then fetch the
                 * desired method.
                 */
                Object logObject = ((Configurable) log).getConfigurationObject();
                logClass = logObject.getClass();
                Method ms = fetchSetLogSize( logObject );


                /*
                 * Start with arbitrary, positive minimum and maximum values.  Select the
                 * mid-point between the two and try that value as the new log size.  If
                 * the value works, use that value as the new minimum; otherwise, use that
                 * value as the new maximum.  When the difference between the minimum and
                 * maximum values is less than or equal to 1, the minimum value is the
                 * maximum number of LogEntry objects that can be stored in a log.
                 */
                Object o;
                int logSizeMin = 1;
                int logSizeMax = Integer.MAX_VALUE;
                int logSize;
                String logSizeString, logSizeMinString, logSizeMaxString;
                while ( true ) {
                    logSize = logSizeMin + (logSizeMax - logSizeMin + 1) / 2;
                    logSizeString = new StringBuffer().append( "         " ).append( logSize ).toString();
                    logSizeString = logSizeString.substring( logSizeString.length() - 10 );
                    logSizeMinString = new StringBuffer().append( "         " ).append( logSizeMin ).toString();
                    logSizeMinString = logSizeMinString.substring( logSizeMinString.length() - 10 );
                    logSizeMaxString = new StringBuffer().append( "         " ).append( logSizeMax ).toString();
                    logSizeMaxString = logSizeMaxString.substring( logSizeMaxString.length() - 10 );
                    System.out.print(  "Min: " + logSizeMinString +
                                       "  Max: " + logSizeMaxString +
                                      "  Trying: " + logSizeString );
                    try {
                        o = ms.invoke( logObject, new Object[] { new Integer( logSize ) } );
                        logSizeMin = logSize;
                        System.out.println();
                        if ((logSizeMax - logSizeMin) <= 1) {
                            System.out.println( "\nUp to " + logSizeMin + " LogEntry objects may be stored in this log." );
                            break;
                        }
                    } catch (InvocationTargetException e) {
                        System.out.println( "  " + e.getTargetException().getMessage() );
                        logSizeMax = logSize;
                    }
                }
            } else {
                System.err.println( "This log does not support the Configurable interface." );
                return;
            }

        } catch (Exception e) {
           System.err.println( e.getMessage() );
        }

    }

    /* Get the Log service reference.  This is synchronized, just in case
     * there are attempts to uninstall the Log service itself from the
     * framework.
     */
    public final static String LOG_CLASS = "org.osgi.service.log.LogService";
    private LogService log = null;  // Reference to the Log service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
        }
    }

    /*
     * Fetch the setLogSize() method.
     */
    private Method fetchSetLogSize( Object logObject ) throws NoSuchMethodException {
        Method m = logClass.getMethod( "setLogSize", new Class[] { int.class } );  // method, arguments
        // JDK 1.2: m.setAccessible( true );
        return m;
    }



}
