/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * LogListening
 *
 * Set up a listener that hears messages as they are being logged and
 * prints them when they are logged.  Some sample messages are logged.
 * after that, this service will continue to run and listen to other
 * logged messages, including all Framework events.
 *
 * There is a critical, brief delay following the addition of the
 * LogListener.  This is necessary to allow the FrameworkEvent.STARTED
 * event to be heard before the LogListener is shut down.
 *
 */

package loglistening;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogEntry;
import org.osgi.service.log.LogListener;
import org.osgi.service.log.LogService;
import org.osgi.service.log.LogReaderService;

public class LogListening implements LogListener {

    LogListening(BundleContext bc) {
        try {

            // Use the bundle context to create the Log service reference.
            getServices( bc );
            if (log == null) {
                System.err.println( "Unable to get Log service reference!" );
                return;
            }
            if (logreader == null) {
                System.err.println( "Unable to get LogReader service reference!" );
                return;
            }

            /* Set up the listener so we can hear messages as they are being
             * logged.  Then generate some messages that should be printed.
             */
            logreader.addLogListener( this );
            Thread.sleep( 10 );   // Allow Framework events to be heard first

            System.out.println( "Using: public void log(int level, String msg)" );
            log.log( LogService.LOG_ERROR,   "Sample LOG_ERROR" );
            log.log( LogService.LOG_WARNING, "Sample LOG_WARNING" );
            log.log( LogService.LOG_INFO,    "Sample LOG_INFO" );
            log.log( LogService.LOG_DEBUG,   "Sample LOG_DEBUG" );

            /* Remove the log listener so that we can't hear messages anymore.
             * All logged messages that follow should not be printed.
             */
            logreader.removeLogListener( this );
            log.log(LogService.LOG_ERROR,   "ERROR: You shouldn't see this");

            /* Establish the listener again as a permanent listener.  This
             * is done for testing purposes and really should be paired with
             * the removal of the listener when the bundle is finished executing.
             */
            logreader.addLogListener( this );

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /*
     * Get references to the Log and LogReader services.
     */
    private final static String LOG_CLASS = "org.osgi.service.log.LogService";
    private final static String LOG_READER_CLASS = "org.osgi.service.log.LogReaderService";
    private LogService log = null;                // Reference to the Log service object
    private LogReaderService logreader = null;    // Reference to the LogReader service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
            if ((sr = bc.getServiceReference( LOG_READER_CLASS )) != null) {
                logreader = (LogReaderService) bc.getService( sr );
            }
        }
    }

    /*
     * LogListener implementation.
     *
     * Intercept Log messages here as they are logged and print them to the console.
     */
    public void logged( LogEntry entry ) {
        try {
          System.out.println( entry.getMessage() );
        } catch (Exception e) {
            System.err.println( e.getMessage() );
            return;
        }
    }

}
