/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved. 
 * 
 * This software is the confidential and proprietary information of Sun 
 * Microsystems, Inc. ("Confidential Information"). You shall not 
 * disclose such Confidential Information and shall use it only in 
 * accordance with the terms of the license agreement you entered into 
 * with Sun. 
 *
 * 
 * LogReflect
 *
 * An example of how to use reflection to reveal the Configurable
 * methods within the Log service.  Other information about an object
 * is also revealed, just to demonstrate reflection itself.
 *
 * The four Configurable methods supported within the Log service are
 * invoked using reflection in order to demonstrate how reflection can
 * be used to configure the Log service.
 * 
 */

package logreflect;

import org.osgi.framework.BundleContext;
import org.osgi.framework.Configurable;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogService;
import java.lang.NoSuchMethodException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;

public class LogReflect {

    // Lead character, representing an array
    private static final char ARRAY_INDICATOR = '[';

    // String version of above
    private static final String ARRAY_INDICATOR_STRING =
            new Character( ARRAY_INDICATOR ).toString();

    // Tab-over string
    private static final String TAB_STRING = "\n    ";

    LogReflect( BundleContext bc ) {
        try {

            // Use the bundle context to create the Log service reference.
            getServices( bc );
            if (log == null) {
                System.err.println( "Unable to get the Log service reference!" );
                return;
            }

            if (log instanceof Configurable) {

                /*
                 * Get the LogConfigurationObject and its class.
                 */
                Object logObject = ((Configurable) log).getConfigurationObject();
                Class logClass = logObject.getClass();

                /*
                 * This part can be omitted and simply snoops for information
                 * about a class.  It will work on any class, but be prepared
                 * for lots of output if a class with lots of methods and lots
                 * of inheritance is tested here.
                 *
                 */
                System.out.println( "Class Name: " + TAB_STRING + logClass.getName() );
                System.out.println( "Modifiers: " + getModifiers( logClass ) );
                System.out.println( "Superclasses: " + getSuperclasses( logClass ) );
                System.out.println( "Interfaces: " + getInterfaces( logClass ) );
                System.out.println( "Fields: " + getFields( logClass ) );
                System.out.println( "Constructors: " + getConstructors( logClass ) );
                System.out.println( "Methods: " + getMethods( logClass ) );
                System.out.println( "Declared Classes and Interfaces: " + getDeclaredClasses( logClass ) );
                System.out.println( "Declared Fields: " + getDeclaredFields( logClass ) );
                System.out.println( "Declared Methods: " + getDeclaredMethods( logClass ) );


                /*
                 * These next parts demonstrate uses of reflection that bundles
                 * would use to configure parts of the Log service.  The exception
                 * handling for InvocationTargetException will show messages for
                 * exceptions that happen in the individual methods when bad values
                 * are passed into those methods.  You can experiment with negative
                 * values, values of zero and large values to see what happens.
                 *
                 */

                // Variable used in the next parts
                String methodName = null;


                // Use reflection to get the size of the log
                methodName = "getLogSize";
                System.out.println( "\nMethod: " + methodName );
                try {

                    // See if the method() exists
                    Method m = logClass.getMethod( methodName, new Class[] {} );

                    // Invoke the method
                    Object o = m.invoke( logObject, new Object[] {} );
                    int size = ((Integer) o).intValue();
                    System.out.println( "Current Log size = "  + size + " LogEntry elements." );

                } catch (NoSuchMethodException e) {
                    System.err.println( "Sorry, " + methodName + " doesn't exist." );
                } catch (InvocationTargetException e) {
                    System.err.println( e.getTargetException().getMessage() );
                } catch (Exception e) {
                    System.err.println( "ERROR: " + e.getMessage() );
                }


                // Use reflection to set the size of the log
                methodName = "setLogSize";
                System.out.println( "\nMethod: " + methodName );
                try {

                    // See if method( int ) exists
                    Method m = logClass.getMethod( methodName, new Class[] { int.class } );

                    // Invoke the method
                    int newSize = 50;  // Arbitrary size of LogEntry objects
                    Object o = m.invoke( logObject, new Object[] { new Integer( newSize ) } );
                    System.out.println( "Size should be set to " + newSize );

                } catch (NoSuchMethodException e) {
                    System.err.println( "Sorry, " + methodName + " doesn't exist." );
                } catch (InvocationTargetException e) {
                    System.err.println( e.getTargetException().getMessage() );
                } catch (Exception e) {
                    System.err.println( "ERROR: " + e.getMessage() );
                }


                // Use reflection to get the size of the log
                methodName = "getLogSize";
                System.out.println( "\nMethod: " + methodName );
                try {

                    // See if the method() exists
                    Method m = logClass.getMethod( methodName, new Class[] {} );

                    // Invoke the method
                    Object o = m.invoke( logObject, new Object[] {} );
                    int size = ((Integer) o).intValue();
                    System.out.println( "New Log size = "  + size + " LogEntry elements." );

                } catch (NoSuchMethodException e) {
                    System.err.println( "Sorry, " + methodName + " doesn't exist." );
                } catch (InvocationTargetException e) {
                    System.err.println( e.getTargetException().getMessage() );
                } catch (Exception e) {
                    System.err.println( "ERROR: " + e.getMessage() );
                }


                // Use reflection to get the severity threshold level
                methodName = "getSeverityThreshold";
                System.out.println( "\nMethod: " + methodName );
                try {

                    // See if the method() exists
                    Method m = logClass.getMethod( methodName, new Class[] {} );

                    // Invoke the method
                    Object o = m.invoke( logObject, new Object[] {} );
                    int threshold = ((Integer) o).intValue();
                    System.out.println( "Current severity threshold = "  + threshold );

                } catch (NoSuchMethodException e) {
                    System.err.println( "Sorry, " + methodName + " doesn't exist." );
                } catch (InvocationTargetException e) {
                    System.err.println( e.getTargetException().getMessage() );
                } catch (Exception e) {
                    System.err.println( "ERROR: " + e.getMessage() );
                }


                // Use reflection to set the severity threshold level
                methodName = "setSeverityThreshold";
                System.out.println( "\nMethod: " + methodName );
                try {

                    // See if method( int ) exists
                    Method m = logClass.getMethod( methodName, new Class[] { int.class } );

                    // Invoke the method
                    int newThreshold = 2;  // Arbitrary threshold level
                    Object o = m.invoke( logObject, new Object[] { new Integer( newThreshold ) } );
                    System.out.println( "Severity threshold should be set to " + newThreshold );

                } catch (NoSuchMethodException e) {
                    System.err.println( "Sorry, " + methodName + " doesn't exist." );
                } catch (InvocationTargetException e) {
                    System.err.println( e.getTargetException().getMessage() );
                } catch (Exception e) {
                    System.err.println( "ERROR: " + e.getMessage() );
                }


                // Use reflection to get the severity threshold level
                methodName = "getSeverityThreshold";
                System.out.println( "\nMethod: " + methodName );
                try {

                    // See if the method() exists
                    Method m = logClass.getMethod( methodName, new Class[] {} );

                    // Invoke the method
                    Object o = m.invoke( logObject, new Object[] {} );
                    int threshold = ((Integer) o).intValue();
                    System.out.println( "New severity threshold = "  + threshold );

                } catch (NoSuchMethodException e) {
                    System.err.println( "Sorry, " + methodName + " doesn't exist." );
                } catch (InvocationTargetException e) {
                    System.err.println( e.getTargetException().getMessage() );
                } catch (Exception e) {
                    System.err.println( "ERROR: " + e.getMessage() );
                }


                // Demonstrate what happens with an unknown method name
                methodName = "fooMethod";
                System.out.println( "\nMethod: " + methodName );
                try {

                    // See if the method, with no arguments, exists
                    Method m = logClass.getMethod( methodName, new Class[] {} );

                    // Invoke the method
                    Object o = m.invoke( logObject, new Object[] {} );
                    int size = ((Integer) o).intValue();
                    System.out.println( "Log size (in LogEntry elements) = "  + size );

                } catch (NoSuchMethodException e) {
                    System.err.println( "Sorry, " + methodName + " doesn't exist." );
                } catch (InvocationTargetException e) {
                    System.err.println( e.getTargetException().getMessage() );
                } catch (Exception e) {
                    System.err.println( "ERROR: " + e.getMessage() );
                }

            } else {
                System.err.println( "Log is not configurable!" );
            }

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /* Get the Log service reference.  This is synchronized, just in case
     * there are attempts to uninstall the Log service itself from the
     * framework.
     */
    public final static String LOG_CLASS = "org.osgi.service.log.LogService";
    private LogService log = null;  // Reference to the Log service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
        }
    }


    /*
     * Get the modifier for the class: public, abstract or final
     */
    private String getModifiers( Class theClass ) {
        int modifier = theClass.getModifiers();
        StringBuffer sb = new StringBuffer( TAB_STRING );

        if (Modifier.isPublic( modifier )) {
            sb.append( "public " );
        }
        if (Modifier.isFinal( modifier )) {
            sb.append( "final " );
        }
        if (Modifier.isAbstract( modifier )) {
            sb.append( "abstract " );
        }

        return sb.toString();
    }

    /*
     * Find all of the superclasses for the given subclass
     */
    private String getSuperclasses( Class subclass ) {
        Class superclass = null;
        StringBuffer sb = new StringBuffer();

        while ((superclass = subclass.getSuperclass()) != null) {
            sb.append( TAB_STRING + superclass.getName() );
            subclass = superclass;
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Get the interfaces for the given class
     */
    private String getInterfaces( Class theClass ) {
        Class[] theInterfaces = theClass.getInterfaces();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < theInterfaces.length; i++) {
            sb.append( TAB_STRING + theInterfaces[i].getName() );
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Get the field names for the given class
     */
    private String getFields( Class theClass ) {
        Field[] theFields = theClass.getFields();
        String fieldName = null;
        String fieldType = null;
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < theFields.length; i++) {
            fieldType = theFields[i].getType().getName();
            fieldName = theFields[i].getName();
            sb.append( TAB_STRING + getParameterType( fieldType ) + " " + fieldName );
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Get the names of the constructors for the given class
     */
    private String getConstructors( Class theClass ) {
        Constructor[] theConstructors = theClass.getConstructors();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < theConstructors.length; i++) {
            Class[] parameterTypes = theConstructors[i].getParameterTypes();
            sb.append( TAB_STRING + "(" );
            for (int j = 0; j < parameterTypes.length; j++) {
                if (j > 0) {
                    sb.append( "," );
                }
                sb.append( " " + getParameterType( parameterTypes[j].getName() ) );
            }
            if (parameterTypes.length > 0) {
                sb.append( " " );
            }
            sb.append( ")" );
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Get the method information for the given class
     */
    private String getMethods( Class theClass ) {
        Class superclass = theClass.getSuperclass();
        Method[] theMethods = theClass.getMethods();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < theMethods.length; i++) {
            Class[] parameterTypes = theMethods[i].getParameterTypes();
            sb.append( TAB_STRING + 
                       getParameterType( theMethods[i].getReturnType().getName() ) +
                       " " + theMethods[i].getName() + "(" );
            for (int j = 0; j < parameterTypes.length; j++) {
                if (j > 0) {
                    sb.append( "," );
                }
                sb.append( " " + getParameterType( parameterTypes[j].getName() ) );
            }
            if (parameterTypes.length > 0) {
                sb.append( " " );
            }
            sb.append( ")" );
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Get the declared classes for the given class
     */
    private String getDeclaredClasses( Class theClass ) {
        Class[] theClasses = theClass.getDeclaredClasses();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < theClasses.length; i++) {
            sb.append( TAB_STRING + theClasses[i].getName() );
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Get the declared fields for the given class
     */
    private String getDeclaredFields( Class theClass ) {
        Field[] theFields = theClass.getDeclaredFields();
        String fieldName = null;
        String fieldType = null;
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < theFields.length; i++) {
            fieldType = theFields[i].getType().getName();
            fieldName = theFields[i].getName();
            sb.append( TAB_STRING + getParameterType( fieldType ) + " " + fieldName );
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Get the declared methods for the given class
     */
    private String getDeclaredMethods( Class theClass ) {
        Method[] theMethods = theClass.getDeclaredMethods();
        StringBuffer sb = new StringBuffer();

        for (int i = 0; i < theMethods.length; i++) {
            Class[] parameterTypes = theMethods[i].getParameterTypes();
            sb.append( TAB_STRING +
                       getParameterType( theMethods[i].getReturnType().getName() ) +
                       " " + theMethods[i].getName() + "(" );
            for (int j = 0; j < parameterTypes.length; j++) {
                if (j > 0) {
                    sb.append( "," );
                }
                sb.append( " " + getParameterType( parameterTypes[j].getName() ) );
            }
            if (parameterTypes.length > 0) {
                sb.append( " " );
            }
            sb.append( ")" );
        }

        if (sb.length() > 0) {
            return sb.toString();
        }
        return (TAB_STRING + "None.");
    }

    /*
     * Convert the Java signature form (if any) for a parameter to
     * String form and append it to the given StringBuffer.
     */
    private String getParameterType( String typeInfo ) {
        StringBuffer sb = new StringBuffer();

        if (typeInfo.startsWith( ARRAY_INDICATOR_STRING )) {
            int lastIndex = typeInfo.lastIndexOf( ARRAY_INDICATOR );

            switch ( typeInfo.toCharArray()[lastIndex+1] ) {
                case 'B':
                    sb.append( byte.class.getName() );
                    break;
                case 'C':
                    sb.append( char.class.getName() );
                    break;
                case 'D':
                    sb.append( double.class.getName() );
                    break;
                case 'F':
                    sb.append( float.class.getName() );
                    break;
                case 'I':
                    sb.append( int.class.getName() );
                    break;
                case 'J':
                    sb.append( long.class.getName() );
                    break;
                case 'L':
                    sb.append( typeInfo.substring( lastIndex+2, typeInfo.length()-1 ) );
                    break;
                case 'S':
                    sb.append( short.class.getName() );
                    break;
                case 'Z':
                    sb.append( boolean.class.getName() );
                    break;
                default:
                    sb.append( "unknown" );
                    break;
            }
            for (int index = 0; index <= lastIndex; index++) {
                sb.append( "[]" );
            }
        } else {
            sb.append( typeInfo );
        }
        return sb.toString();
    }

}
