/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * LogSorting
 *
 * Log some messages with random severities.  Then generate an array
 * of those entries that can be sorted, based on the severities.  The
 * use of the array allows a Comparator to be used for sorting.
 *
 * This example relied on the JDK 1.2.x classes Comparator and Arrays,
 * but was modified to be pJava-compliant (i.e., JDK 1.1.x-compliant).
 * The Comparator interface has been re-created here so that if this
 * example is ever used in a JDK 1.2.x environment (or above), the
 * number of changes would be trivial (i.e., remove the call to the
 * mergeSort() algorithm and enable all "JDK 1.2.x"-commented lines).
 *
 */

package logsorting;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Configurable;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogEntry;
import org.osgi.service.log.LogService;
import org.osgi.service.log.LogReaderService;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Date;
// JDK 1.2.x: import java.util.Comparator;
import java.util.Enumeration;
import java.util.Random;
import java.util.Vector;
// JDK 1.2.x: import java.util.Arrays;


public class LogSorting implements Comparator {

    // Maximum size of the log
    private static final int MAX_LOG_SIZE = 50;

    // The total number of messages to log/sort
    private static final int DEFAULT_MAX_MESSAGES = 25;

    // The prefix for all logged messages from this bundle.
    private static final String MESSAGE_PREFIX = "Message #";

    LogSorting( BundleContext bc ) {
        try {

            // Use the bundle context to create the Log service reference.
            getServices( bc );
            if (log == null) {
                System.err.println( "Unable to get Log service reference!" );
                return;
            }
            if (logreader == null) {
                System.err.println( "Unable to get LogReader service reference!" );
                return;
            }


            // Try to double the size of the log; otherwise, use the default size
            int maxMessages = DEFAULT_MAX_MESSAGES;
            if (log instanceof Configurable) {

                /*
                 * Get the logConfigurationObject and its type (class).  Then fetch the
                 * desired method.
                 */
                Object logObject = ((Configurable) log).getConfigurationObject();
                Class logClass = logObject.getClass();
                Method mg = fetchGetLogSize( logClass );
                Method ms = fetchSetLogSize( logClass );

                // Get the current size of the log
                Object o = mg.invoke( logObject, new Object[] {} );
                int logSize = ((Integer) o).intValue();
                System.out.println( "Log size: "  + logSize );

                // Attempt to double the size of the log
                int newSize = logSize * 2;
                if (newSize >= MAX_LOG_SIZE) {
                    newSize = logSize;
                }
                System.out.println( "New size: " + newSize );
                try {
                    o = ms.invoke( logObject, new Object[] { new Integer( newSize ) } );
                    maxMessages = newSize;
                } catch (Exception e) {
                    System.out.println( "Unable to set log size to " + newSize + "." +
                                        "  Using default of " + logSize + "." );
                }

            }


            // Fill the log with a bunch of messages of random severity levels.
            int severities[] = new int[] { LogService.LOG_ERROR,
                                           LogService.LOG_WARNING,
                                           LogService.LOG_INFO,
                                           LogService.LOG_DEBUG
                                         };
            Random randomizer = new Random();
            int rndLimit = severities.length;
            int index = 0;
            for (int i = 1; i <= maxMessages; i++) {
                index = Math.abs( randomizer.nextInt() ) % rndLimit;
                log.log( severities[ index ], MESSAGE_PREFIX + i );
            }
            randomizer = null;

            // Create a vector of entries, then an array of entries
            Vector v = new Vector();
            for( Enumeration e = logreader.getLog(); e.hasMoreElements(); ) {
                v.addElement( (LogEntry) e.nextElement() );
            }
            Object[] loggedEntries = v.toArray();
            v = null;

            // Show the unsorted list
            System.out.println( "\nUnsorted:" );
            showLogEntries( loggedEntries );

            // Sort the array.
            // JDK 1.2.x:  Arrays.sort( loggedEntries, this );
            mergeSort( loggedEntries, this );

            // Show the new, sorted set of Log entries.
            System.out.println( "Sorted:" );
            showLogEntries( loggedEntries );

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /*
     * Get references to the Log and LogReader services.
     */
    private final static String LOG_CLASS = "org.osgi.service.log.LogService";
    private final static String LOG_READER_CLASS = "org.osgi.service.log.LogReaderService";
    private LogService log = null;                // Reference to the Log service object
    private LogReaderService logreader = null;    // Reference to the LogReader service object
    ServiceReference sr = null;                   // Service reference for Log
    synchronized void getServices( BundleContext bc ) {
        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
            if ((sr = bc.getServiceReference( LOG_READER_CLASS )) != null) {
                logreader = (LogReaderService) bc.getService( sr );
            }
        }
    }

    /*
     * See if getLogSize() is available (An exception will be thrown if the method
     * or its parameters don't match).
     *
     * IMPORTANT!!
     *   The method is normally inaccessible, so override this fact.  If access is
     *   not made true, an IllegalAccessException will be thrown, since
     *   logConfigurationObject is a private class...
     */
    private Method fetchGetLogSize( Class logClass ) throws NoSuchMethodException {
        Method m = null;
        m = logClass.getMethod( "getLogSize", new Class[] {} );  // method, arguments (none)
        // JDK 1.2: m.setAccessible( true );
        return m;
    }

    /*
     * Fetch the setLogSize() method.
     */
    private Method fetchSetLogSize( Class logClass ) throws NoSuchMethodException {
        Method m = null;
        m = logClass.getMethod( "setLogSize", new Class[] { int.class } );  // method, arguments
        // JDK 1.2: m.setAccessible( true );
        return m;
    }


    /*
     *  Show all log entries
     */
    public void showLogEntries( Object[] entries ) {
        for( int i = 0; i < entries.length; i++ ) {
            LogEntry entry = (LogEntry) entries[i];
            System.out.println( getLevelString( entry.getLevel() ) +
                                ": " + entry.getMessage() );
        }
        System.out.println();
    }

    /*
     * showLevel
     *
     * Shows the severity level in String form.
     */
    public String getLevelString( int level ) {
        String s = null;
        switch( level ) {
            case LogService.LOG_ERROR:
                s = "  ERROR";
                break;
            case LogService.LOG_WARNING:
                s = "WARNING";
                break;
            case LogService.LOG_INFO:
                s = "   INFO";
                break;
            case LogService.LOG_DEBUG:
                s = "  DEBUG";
                break;
            default:
                s = "UNKNOWN";
                break;
        }
        return s;
    }

    /*
     * Comparator implementation
     *
     * Entries are sorted according to their log severity level.
     *
     * Only compare() is implemented.  Since logged entries are retrieved in reverse
     * order, items that are the same are forcibly swapped so that they will appear
     * in ascending rather than descending order.
     *
     */
    public int compare( Object o1, Object o2 ) {
        int level1 = ((LogEntry) o1).getLevel();     // The severity level for o1
        int level2 = ((LogEntry) o2).getLevel();     // The severity level for o2

        // Get the status for o1 and forcibly swap objects of the same level
        int status = level1 - level2;
        if (status == 0) {
            status = 1;
        }
        return status;
    }


    /*
     * mergeSort
     *
     * A generic merge-sort algorithm that operates on an array of
     * objects.  The array is sorted using a divide-and-conquer
     * approach:
     *
     * Divide:
     * The array is divided if the number of entries is smaller than a
     * threshold limit.  The smaller array is then sorted again using
     * recursion.
     *
     * Conquer:
     * The smaller, sorted arrays are re-combined into the solution (A
     * sorted array of objects).
     *
     */
    public void mergeSort( Object[] sequence, Comparator c ) {

        // n, the number of elements in the sequence
        int n = sequence.length;

        // Sequences containing 0 or 1 element are already sorted
        if (n < 2) {
            return;
        }

        // Divide the sequence into two half-sequences, s1 and s2
        int index = 0, index1 = 0, index2 = 0;
        Object[] s1 = new Object[ (n + 1 ) / 2 ];
        for (int i = 1; i <= ((n + 1) / 2); i++) {
            s1[ index1++ ] = sequence[ index++ ];
        }
        Object[] s2 = new Object[ n / 2 ];
        for (int i = 1; i <= (n / 2); i++) {
            s2[ index2++ ] = sequence[ index++ ];
        }

        // Recurse on each half
        mergeSort( s1, c );
        mergeSort( s2, c );

        // Conquer (Bring the sequence fragments into line)
        merge( s1, s2, c, sequence );
    }

    /*
     * Merge two sequences into a single sequence.  When the sequences
     * differ in length, merge as much of the shorter sequence with the
     * longer one; then copy the rest to the final sequence.
     */
    public void merge( Object[] s1, Object[] s2,
            Comparator c, Object[] sequence ) {
        int n1 = s1.length;
        int n2 = s2.length;
        int index1 = 0, index2 = 0, index = 0;

        // Merge elements until one or both sequences is used up.
        while ((index1 < n1) && (index2 < n2)) {
            if (c.compare( s1[ index1 ], s2[ index2 ] ) < 0) {
                sequence[ index++ ] = s1[ index1++ ];
            } else {
                sequence[ index++ ] = s2[ index2++ ];
            }
        }

        // Copy second sequence over if the first one is empty.
        if (index1 >= n1) {
            while (index2 < n2) {
                sequence[ index++ ] = s2[ index2++ ];
            }
        }

        // Copy first sequence over if the second one is empty.
        if (index2 >= n2) {
            while (index1 < n1) {
                sequence[ index++ ] = s1[ index1++ ];
            }
        }
    }
}
