/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * LogTime
 *
 * Demonstrate how to use the Log, LogReader and Timer services to log
 * the time in milliseconds since January 1, 1970, 00:00:00 GMT, but
 * formatted in a particular DateFormat style for 'this' locale as well
 * as other locales.
 *
 */

package logtime;

import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Configurable;
import org.osgi.framework.ServiceReference;
import org.osgi.service.log.LogEntry;
import org.osgi.service.log.LogListener;
import org.osgi.service.log.LogReaderService;
import org.osgi.service.log.LogService;
import com.sun.jes.service.timer.TimerService;
import com.sun.jes.service.timer.TimerTask;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.text.DateFormat;
import java.util.Date;
import java.util.Locale;

public class LogTime implements LogListener {

    // Prefix used to distinguish time messages from other logged messages
    private static final String TIME_PREFIX = "TIME:";

    LogTime( BundleContext bc ) {
        try {

            // Use the bundle context to create the Log service reference.
            getServices( bc );
            if (log == null) {
                System.err.println( "Unable to get Log service reference!" );
                return;
            }
            if (logreader == null) {
                System.err.println( "Unable to get LogReader service reference!" );
                return;
            }
            if (timer == null) {
                System.err.println( "Unable to get Timer service reference!" );
                return;
            }


            // Add the listener to the time messages
            logreader.addLogListener( this );


            // Show any messages before starting the task
            System.out.println( "\n" + "Press Return to stop this example" + "\n" );


            // Select the time format for 'this' locale.
            final int theDateFormat = DateFormat.LONG;
            final DateFormat timeFormat =
                    DateFormat.getTimeInstance( theDateFormat );
            final DateFormat timeFormatFrance =
                    DateFormat.getTimeInstance( theDateFormat, Locale.FRANCE );

            /*
             * Schedule a task which prints out the time, formatted for
             * 'this' locale.  The task will begin shortly and then repeat
             * every 1000ms.
             */
            TimerTask task = timer.schedule( new Runnable() {
                public void run() {
                    log.log( LogService.LOG_INFO, TIME_PREFIX +
                            "Local: " + timeFormat.format( new Date() ) + "  " +
                            "France: " + timeFormatFrance.format( new Date() ) + "  "
                    );
                }
            } , 750, 1000 );


            /*
             * Set up the buffered input reader for the console and then
             * wait for the user to press the Return key.
             */
            BufferedReader console = new BufferedReader(
                    new InputStreamReader( System.in ) );
            String input = console.readLine();


            // Cancel all tasks for this bundle
            timer.cancel();
            timer = null;

            // Discard the Log listener.
            logreader.removeLogListener( this );


        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /*
     * Get references to the Log and Timer services.
     */
    private final static String LOG_CLASS = "org.osgi.service.log.LogService";
    private final static String LOG_READER_CLASS = "org.osgi.service.log.LogReaderService";
    private final static String TIMER_CLASS = "com.sun.jes.service.timer.TimerService";
    private LogService log = null;                // Reference to the Log service object
    private LogReaderService logreader = null;    // Reference to the LogReader service object
    private TimerService timer = null;            // Reference to the Timer service object
    ServiceReference sr = null;
    synchronized void getServices( BundleContext bc ) {
        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
            if ((sr = bc.getServiceReference( LOG_READER_CLASS )) != null) {
                logreader = (LogReaderService) bc.getService( sr );
            }
        }
        if (timer == null) {
            if ((sr = bc.getServiceReference( TIMER_CLASS )) != null) {
                timer = (TimerService) bc.getService( sr );
            }
        }
    }

    /*
     * LogListener implementation
     */
    public void logged( LogEntry entry ) {
        String message = entry.getMessage();

        if (message.startsWith( TIME_PREFIX ) == true ) {

            // Remove the prefix from the message
            message = message.substring( TIME_PREFIX.length() );

            // Overwrite the current time with this new time
            System.out.print( "\r" + message );
        }
    }


}
