/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * TimerCancelTimer
 *
 * An example of how to cancel all scheduled tasks for a bundle, simply
 * by cancelling the timer itself.  The timer service isn't actually
 * uninstalled, but remains idle in the background until new tasks are
 * scheduled.
 *
 * All six APIs are used to schedule a simple task.  The tasks are
 * allowed to repeat until the timer is cancelled.  Note that the tasks
 * that cannot repeat will execute only once.
 *
 */

package timercanceltimer;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import com.sun.jes.service.timer.TimerService;
import com.sun.jes.service.timer.TimerTask;
import java.util.Date;

public class TimerCancelTimer {

    // Delays (milliseconds)
    private static final long  START_DELAY = 1000;     // Delay before task starts
    private static final long REPEAT_DELAY = 250;      // Delay before task repeats
    private static final long      REPEATS = 3;        // The number of repeats before stopping tasks

    /*
     * Variables used to keep track of the number of executions for each task.
     * These variables must exist as fields in this outer class so that the run()
     * methods can access them.
     */
    private int task1Count = 0;
    private int task2Count = 0;
    private int task3Count = 0;
    private int task4Count = 0;
    private int task5Count = 0;
    private int task6Count = 0;

    TimerCancelTimer( BundleContext bc ) {

        // Use the bundle context to create the Timer service reference.
        getServices( bc );
        if (timer == null) {
            System.err.println( "Unable to get Timer service reference!" );
            return;
        }

        try {

            // Establish the Runnable tasks for each of the APIs
            Runnable runtask1 = new Runnable() {
                public void run() {
                    task1Count++;
                    System.out.println( "Task 1: Execution #" + task1Count );
                }
            };

            Runnable runtask2 = new Runnable() {
                public void run() {
                    task2Count++;
                    System.out.println( "Task 2: Execution #" + task2Count );
                }
            };

            Runnable runtask3 = new Runnable() {
                public void run() {
                    task3Count++;
                    System.out.println( "Task 3: Execution #" + task3Count );
                }
            };

            Runnable runtask4 = new Runnable() {
                public void run() {
                    task4Count++;
                    System.out.println( "Task 4: Execution #" + task4Count );
                }
            };

            Runnable runtask5 = new Runnable() {
                public void run() {
                    task5Count++;
                    System.out.println( "Task 5: Execution #" + task5Count );
                }
            };

            Runnable runtask6 = new Runnable() {
                public void run() {
                    task6Count++;
                    System.out.println( "Task 6: Execution #" + task6Count );
                }
            };

            // Schedule all tasks using each of the APIs.
            System.out.println( "Scheduling all tasks to repeat " + (REPEATS + 1) + " times." );
            TimerTask task1 = timer.schedule(
                    runtask1, new Date(new Date().getTime() + START_DELAY) );
            TimerTask task2 = timer.schedule(
                    runtask2, new Date(new Date().getTime() + START_DELAY), REPEAT_DELAY );
            TimerTask task3 = timer.schedule(
                    runtask3, START_DELAY );
            TimerTask task4 = timer.schedule(
                    runtask4, START_DELAY, REPEAT_DELAY );
            TimerTask task5 = timer.scheduleAtFixedRate(
                    runtask5, new Date(new Date().getTime() + START_DELAY), REPEAT_DELAY);
            TimerTask task6 = timer.scheduleAtFixedRate(
                    runtask6, START_DELAY, REPEAT_DELAY);

            /*
             * Delay long enough to enter into the middle of a task.
             * Then delay REPEATS times to let the task repeat a few
             * times before the timer is cancelled.
             */
            Thread.sleep( START_DELAY + (REPEAT_DELAY / 2) + (REPEAT_DELAY * REPEATS) );

            // Cancel the timer, which should cancel all of our tasks
            System.out.println( "Cancelling timer and all tasks" );
            timer.cancel();

            // Immediately save the number of times the tasks were executed
            int oldTask1Count = task1Count;
            int oldTask2Count = task2Count;
            int oldTask3Count = task3Count;
            int oldTask4Count = task4Count;
            int oldTask5Count = task5Count;
            int oldTask6Count = task6Count;

            // Delay enough to allow any task to repeat
            Thread.sleep( REPEAT_DELAY );

            // Check to see if tasks executed and if they repeated
            System.out.println( "Verifying that tasks were stopped." );

            if (task1Count == 0) {
                System.err.println( "Task 1 failed to execute" );
            } else if (task1Count != oldTask1Count) {
                System.err.println( "Task 1 didn't execute 1X" );
            } else {
                System.out.println( "Task 1 successfully stopped." );
            }

            if (task2Count == 0) {
                System.err.println( "Task 2 failed to execute" );
            } else if (task2Count != oldTask2Count) {
                System.err.println( "Task 2 executed " + task2Count + " of " + REPEATS + " times" );
            } else {
                System.out.println( "Task 2 successfully stopped." );
            }

            if (task3Count == 0) {
                System.err.println( "Task 3 failed to execute" );
            } else if (task3Count != oldTask3Count) {
                System.err.println( "Task 3 didn't execute 1X" );
            } else {
                System.out.println( "Task 3 successfully stopped." );
            }

            if (task4Count == 0) {
                System.err.println( "Task 4 failed to execute" );
            } else if (task4Count != oldTask4Count) {
                System.err.println( "Task 4 executed " + task4Count + " of " + REPEATS + " times" );
            } else {
                System.out.println( "Task 4 successfully stopped." );
            }

            if (task5Count == 0) {
                System.err.println( "Task 5 failed to execute" );
            } else if (task5Count != oldTask5Count) {
                System.err.println( "Task 5 executed " + task5Count + " of " + REPEATS + " times" );
            } else {
                System.out.println( "Task 5 successfully stopped." );
            }

            if (task6Count == 0) {
                System.err.println( "Task 6 failed to execute" );
            } else if (task6Count != oldTask6Count) {
                System.err.println( "Task 6 executed " + task6Count + " of " + REPEATS + " times" );
            } else {
                System.out.println( "Task 6 successfully stopped." );
            }

            System.out.println( "Done!" );

        } catch (Exception e) {
            System.err.println( e.getMessage() );
            return;
        }
    }

    /*
     * Get the Timer service reference.
     */
    public final static String TIMER_CLASS = "com.sun.jes.service.timer.TimerService";
    private TimerService timer = null;
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (timer == null) {
            if ((sr = bc.getServiceReference( TIMER_CLASS )) != null) {
                timer = (TimerService) bc.getService( sr );
            }
        }
    }

}
