/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved. 
 * 
 * This software is the confidential and proprietary information of Sun 
 * Microsystems, Inc. ("Confidential Information"). You shall not 
 * disclose such Confidential Information and shall use it only in 
 * accordance with the terms of the license agreement you entered into 
 * with Sun. 
 *
 * 
 * TimerLog
 *
 * An example use of the Timer service which logs messages at regular
 * intervals.  Two tasks are scheduled to log messages at different
 * times.  Those two tasks are also automatically stopped at different
 * times, depending on the number of messages that have been logged.
 *
 * A LogListener prints the messages as they are logged and also counts
 * the number of messages that have been logged to that this bundle
 * "knows" when to stop each of the tasks.
 *
 * Note that each individual task is stopped, and not the timer itself.
 * If the timer were to be stopped, all scheduled tasks would be halted.
 *
 */

package timerlog;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import com.sun.jes.service.timer.TimerService;
import com.sun.jes.service.timer.TimerTask;
import org.osgi.service.log.LogEntry;
import org.osgi.service.log.LogListener;
import org.osgi.service.log.LogReaderService;
import org.osgi.service.log.LogService;

public class TimerLog implements LogListener {

    private static final int TASK1_MESSAGE_LIMIT = 11;
    private static final int TASK2_MESSAGE_LIMIT = TASK1_MESSAGE_LIMIT + 2;
    private int messageCounter = 0;  // The number of logged messages

    TimerLog( BundleContext bc ) {
      try {

          // Get the service references for: Log, LogReader and Timer
          getServices( bc );
          if (log == null) {
              System.err.println( "Unable to get the Log service reference!" );
              return;
          }
          if (logreader == null) {
              System.err.println( "Unable to get the LogReader service reference!" );
              return;
          }
          if (timer == null) {
              System.err.println( "Unable to get the Timer service reference!" );
              return;
          }

          // Set up the listener for the messages to be logged.
          System.out.println( "Adding Log listener" );
          logreader.addLogListener( this );


          // Task 1: A new Runnable that logs a message.
          Runnable runtask1 = new Runnable() {
              public void run() {
                  log.log(LogService.LOG_INFO, "Task 1: Running");
              }
          };
  
          // Task 2: Another Runnable that logs a different message.
          Runnable runtask2 = new Runnable() {
              public void run() {
                  log.log(LogService.LOG_INFO, "Task 2: -------");
              }
          };

          // Schedule the tasks
          //
          //                                   Task        Delay       Period
          //                                      |            |            |
          TimerTask task1 = timer.schedule( runtask1, (long)  500, (long)  500 );
          TimerTask task2 = timer.schedule( runtask2, (long) 2000, (long) 1500 );


          /* Wait for the maximum number of messages to be logged and then
           * cancel Task 1 when the quota has been reached.
           */
          while (messageCounter < TASK1_MESSAGE_LIMIT) {
              Thread.sleep( 100 );  // delay less than the scheduled period
          }
          task1.cancel();  // Just cancel our task.  Don't cancel everyone elses!!
          System.out.println( messageCounter + " messages have been logged." );
          System.out.println( "That's all for Task 1, folks!" );


          // Wait around a little longer before cancelling Task 2
          while (messageCounter < TASK2_MESSAGE_LIMIT) {
              Thread.sleep( 100 );
          }
          task2.cancel();
          System.out.println( messageCounter + " messages have been logged." );
          System.out.println( "Bye!!" );


          // Remove the log listener
          System.out.println( "Removing Log listener" );
          logreader.removeLogListener( this );

          // Cancel the timer
          System.out.println( "Cancelling the Timer" );
          timer.cancel();
          timer = null;

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }

    }

    /*
     * Get the service references.
     */
    public final static String LOG_CLASS = "org.osgi.service.log.LogService";
    public final static String LOG_READER_CLASS = "org.osgi.service.log.LogReaderService";
    public final static String TIMER_CLASS = "com.sun.jes.service.timer.TimerService";
    private LogService log = null;
    private LogReaderService logreader = null;
    private TimerService timer = null;
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (log == null) {
            if ((sr = bc.getServiceReference( LOG_CLASS )) != null) {
                log = (LogService) bc.getService( sr );
            }
            if ((sr = bc.getServiceReference( LOG_READER_CLASS )) != null) {
                logreader = (LogReaderService) bc.getService( sr );
            }
        }

        if (timer == null) {
            if ((sr = bc.getServiceReference( TIMER_CLASS )) != null) {
                timer = (TimerService) bc.getService( sr );
            }
        }

    }


    /*
     * LogListener implementation.
     *
     * Intercept Log messages here as they are logged and print them to the console.
     * Also count the number of messages that have been logged.
     */
    public void logged( LogEntry entry ) {
        try {
            System.out.println( entry.getMessage() );
            messageCounter++;
        } catch (Exception e) {
            // Ignore exceptions
        }
    }

}
