/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * TimerNeverRun
 *
 * This example demonstrates how tasks can be cancelled before they
 * get a chance to run.  All six Timer APIs are used.
 *
 */

package timerneverrun;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import com.sun.jes.service.timer.TimerService;
import com.sun.jes.service.timer.TimerTask;
import java.util.Date;

public class TimerNeverRun {

    // Delays (milliseconds)
    private static final long  START_DELAY = 500;     // Delay before task begins
    private static final long REPEAT_DELAY = 100;     // Delay before task repeats
    private static final long    END_DELAY = 500;     // Delay before task ends

    // Variables which must be accessible in run()
    private int taskNumber = 0;                       // The number of the current task
    private boolean testSuccessful = true;            // Assume a test passed

    TimerNeverRun(BundleContext bc) {

        // Use the bundle context to create the Timer service reference.
        getServices( bc );
        if (timer == null) {
            System.err.println( "Unable to get Timer service reference!" );
            return;
        }

        try {
           Runnable runtask = null;     // Generic Runnable
           TimerTask task = null;       // Generic TimerTask

           // schedule(Runnable runtask, Date time)
           //
           // Schedule the task for execution at the specified time.
            taskNumber++;
            runtask = new Runnable() {
                public void run() {
                    System.out.println( "Task " + taskNumber + ": running" );
                    testSuccessful = false;  // failed
                }
            };
            System.out.println( "\n Begin: schedule(Runnable runtask, Date time)" );
            System.out.println( " Delay: " + START_DELAY + " ms" );
            task = timer.schedule( runtask, new Date(new Date().getTime() + START_DELAY) );
            Thread.sleep( START_DELAY / 2 );  // Sleep, but wake up before task executes
            System.out.println( "Cancelling Task " + taskNumber );
            task.cancel();
            System.out.println( "   End: schedule(Runnable runtask, Date time)" );
            if (testSuccessful == false) {
                System.err.println( "Task " + taskNumber + " unexpectedly ran" );
                return;
            }
            System.out.println( "Task was successfully cancelled." );


            // schedule(Runnable runtask, Date firstTime, long period)
            //
            // Schedule the task for repeated fixed-delay execution, beginning at the specified time.
            taskNumber++;
            runtask = new Runnable() {
                public void run() {
                    System.out.println( "Task " + taskNumber + ": running" );
                    testSuccessful = false;
                }
            };
            System.out.println( "\n Begin: schedule(Runnable runtask, Date firstTime, long period)" );
            System.out.println( " Delay: " + START_DELAY + " ms" );
            System.out.println( "Period: " + REPEAT_DELAY + " ms" );
            task = timer.schedule( runtask, new Date(new Date().getTime() + START_DELAY), REPEAT_DELAY );
            Thread.sleep( START_DELAY / 2 );  // Sleep, but wake up before task executes
            System.out.println( "Cancelling Task " + taskNumber );
            task.cancel();
            System.out.println( "   End: schedule(Runnable runtask, Date firstTime, long period)" );
            if (testSuccessful == false) {
                System.err.println( "Task " + taskNumber + " unexpectedly ran" );
                return;
            }
            System.out.println( "Task was successfully cancelled." );


            // schedule(Runnable runtask, long delay)
            //
            // Schedule the task for execution after the specified delay.
            taskNumber++;
            runtask = new Runnable() {
                public void run() {
                    System.out.println( "Task " + taskNumber + ": running" );
                    testSuccessful = false;
                }
            };
            System.out.println( "\n Begin: schedule(Runnable runtask, long delay)" );
            System.out.println( " Delay: " + START_DELAY + " ms" );
            task = timer.schedule( runtask, START_DELAY );
            Thread.sleep( START_DELAY / 2 );  // Sleep, but wake up before task executes
            System.out.println( "Cancelling Task " + taskNumber );
            task.cancel();
            System.out.println( "   End: schedule(Runnable runtask, long delay)" );
            if (testSuccessful == false) {
                System.err.println( "Task " + taskNumber + " unexpectedly ran" );
                return;
            }
            System.out.println( "Task was successfully cancelled." );


            // schedule(Runnable runtask, long delay, long period)
            //
            // Schedule the task for repeated fixed-delay execution,
            // beginning after the specified delay.
            taskNumber++;
            runtask = new Runnable() {
                public void run() {
                    System.out.println( "Task " + taskNumber + ": running" );
                    testSuccessful = false;
                }
            };
            System.out.println( "\n Begin: schedule(Runnable runtask, long delay, long period)" );
            System.out.println( " Delay: " + START_DELAY + " ms" );
            System.out.println( "Period: " + REPEAT_DELAY + " ms" );
            task = timer.schedule( runtask, START_DELAY, REPEAT_DELAY );
            Thread.sleep( START_DELAY / 2 );  // Sleep, but wake up before task executes
            System.out.println( "Cancelling Task " + taskNumber );
            task.cancel();
            System.out.println( "   End: schedule(Runnable runtask, long delay, long period)" );
            if (testSuccessful == false) {
                System.err.println( "Task " + taskNumber + " unexpectedly ran" );
                return;
            }
            System.out.println( "Task was successfully cancelled." );


            // scheduleAtFixedRate(Runnable runtask, Date firstTime, long period)
            //
            // Schedule the task for repeated fixed-delay execution,
            // beginning at the specified time.
            taskNumber++;
            runtask = new Runnable() {
                public void run() {
                    System.out.println( "Task " + taskNumber + ": running" );
                    testSuccessful = false;
                }
            };
            System.out.println( "\n Begin: scheduleAtFixedRate(Runnable runtask, Date firstTime, long period)" );
            System.out.println( " Delay: " + START_DELAY + " ms" );
            System.out.println( "Period: " + REPEAT_DELAY + " ms" );
            task = timer.scheduleAtFixedRate( runtask, new Date(new Date().getTime() + START_DELAY), REPEAT_DELAY );
            Thread.sleep( START_DELAY / 2 );  // Sleep, but wake up before task executes
            System.out.println( "Cancelling Task " + taskNumber );
            task.cancel();
            System.out.println( "   End: scheduleAtFixedRate(Runnable runtask, Date firstTime, long period)" );
            if (testSuccessful == false) {
                System.err.println( "Task " + taskNumber + " unexpectedly ran" );
                return;
            }
            System.out.println( "Task was successfully cancelled." );


            // scheduleAtFixedRate(Runnable runtask, long delay, long period)
            //
            // Schedule the task for repeated fixed-rate execution,
            // beginning after the specified delay.
            taskNumber++;
            runtask = new Runnable() {
                public void run() {
                    System.out.println( "Task " + taskNumber + ": running" );
                    testSuccessful = false;
                }
            };
            System.out.println( "\n Begin: scheduleAtFixedRate(Runnable runtask, long delay, long period)" );
            System.out.println( " Delay: " + START_DELAY + " ms" );
            System.out.println( "Period: " + REPEAT_DELAY + " ms" );
            task = timer.scheduleAtFixedRate( runtask, START_DELAY, REPEAT_DELAY );
            Thread.sleep( START_DELAY / 2 );  // Sleep, but wake up before task executes
            System.out.println( "Cancelling Task " + taskNumber );
            task.cancel();
            System.out.println( "   End: scheduleAtFixedRate(Runnable runtask, long delay, long period)" );
            if (testSuccessful == false) {
                System.err.println( "Task " + taskNumber + " unexpectedly ran" );
                return;
            }
            System.out.println( "Task was successfully cancelled." );

        } catch (Exception e) {
            System.err.println( e.getMessage() );
            return;
        }


        // Test the cancel() method and also use it to cancel any runaway tasks
        // and to stop the timer (scheduler) service itself.
        System.out.println( "\nCancelling timer\n" );
        timer.cancel();
    }

    /*
     * Get the Timer service reference.
     */
    public final static String TIMER_CLASS = "com.sun.jes.service.timer.TimerService";
    private TimerService timer = null;  // Reference to the Timer service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (timer == null) {
            if ((sr = bc.getServiceReference( TIMER_CLASS )) != null) {
                timer = (TimerService) bc.getService( sr );
            }
        }
    }

}
