/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * TimerTaskExample
 *
 * Demonstrate how to use the TimerTask APIs in different ways.
 *
 * The cancel() method is used outside the run() method as well as
 * inside of it.  The use of the method inside run() requires the
 * TimerTask object to be declared in the outer class.
 *
 * The scheduledExecutionTime method is used outside and inside the
 * run() method as well.
 *
 * The tasks are scheduled using simple, repeated scheduling. 
 *
 */

package timertaskexample;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import com.sun.jes.service.timer.TimerService;
import com.sun.jes.service.timer.TimerTask;
import java.util.Date;

public class TimerTaskExample {

    // Delays (milliseconds)
    private static final long  START_DELAY = 1000;
    private static final long REPEAT_DELAY =  250;
    private static final long   TASK_DELAY =   15;
    private static final long  PAUSE_DELAY =  500;

    // The maximum times a task is allowed to repeat before being cancelled.
    private static final int MAX_TASK_EXECUTIONS = 4;

    // Maximum delay before it's too late to run the task
    private static final long MAX_TARDINESS = 15;


    /*
     * These variables must be present in the outer class so that they
     * will be accessible by the run() methods of the inner classes
     */
    int taskCount = 0;         // The number of times the task has executed
    TimerTask task = null;
    long elapsedTime = 0;       // The elapsed time (ms) for a task


    TimerTaskExample( BundleContext bc ) {

        // Use the bundle context to create the Timer service reference.
        getServices( bc );
        if (timer == null) {
            System.err.println( "Unable to get the Timer service reference!" );
            return;
        }

        try {


            /*
             * Schedule a task and delay to let it repeat before cancelling it.
             * Note how task1 can be declared here, since it is not being used
             * within the run() method.
             */
            System.out.println( "Scheduling Task 1" );
            TimerTask task1 = timer.schedule( new Runnable() {
                public void run() {
                    System.out.println( "Task 1 is running" );
                }
            }, START_DELAY, REPEAT_DELAY );
            Thread.sleep( START_DELAY + (REPEAT_DELAY * (MAX_TASK_EXECUTIONS + 1/2)) );

            System.out.print( "Cancelling Task 1..." );
            task1.cancel();
            System.out.println( "Done!\n" );

            Thread.sleep( PAUSE_DELAY );


            /*
             * Schedule a task that can cancel itself after a few runs.  In this
             * example, task must be declared as a variable in the outer class,
             * since run() can only access task's cancel() method this way.
             */
            System.out.println( "Scheduling Task 2" );
            taskCount = 0;
            task = timer.schedule( new Runnable() {
                public void run() {
                    System.out.println( "Task 2 is running" );
                    taskCount++;
                    if (taskCount >= MAX_TASK_EXECUTIONS) {
                        System.out.print( "Cancelling Task 2..." );
                        task.cancel();
                        System.out.println( "Done!" );
                    }
                }
            }, START_DELAY, REPEAT_DELAY );
            Thread.sleep( START_DELAY + (REPEAT_DELAY * (MAX_TASK_EXECUTIONS + 1/2)) );

            // Note: This should not be needed, but is here for safety
            System.out.print( "Cancelling Task 2 (Safety)..." );
            task.cancel();
            System.out.println( "Done!\n" );

            Thread.sleep( PAUSE_DELAY );


            /*
             * Schedule a repeating task and show the elapsed time since the task's
             * scheduled execution time, at TASK_DELAY intervals.  The time must be
             * shown outside of the run() method, since that method can't access task3.
             */
            System.out.print( "Scheduling Task 3  " );
            TimerTask task3 = timer.schedule( new Runnable() {
                public void run() {
                    System.out.println( "\n\nTask 3 is running.  Elapsed times before next execution:" );
                }
            }, START_DELAY, REPEAT_DELAY );
            Thread.sleep( START_DELAY );

            long currentTime = System.currentTimeMillis();
            long endingTime = currentTime + (REPEAT_DELAY * (MAX_TASK_EXECUTIONS + 1/2));
            //long lastElapsedTime = 0;
            do {
                elapsedTime = System.currentTimeMillis() - task3.scheduledExecutionTime();
                if (elapsedTime >= 0) {
                    System.out.print( elapsedTime + "  " );
                }
                Thread.sleep( TASK_DELAY );  // Let the task run for a while
            } while (System.currentTimeMillis() < endingTime);

            System.out.print( "\nCancelling Task 3..." );
            task3.cancel();
            System.out.println( "Done!\n" );

            Thread.sleep( PAUSE_DELAY );


            /*
             * Schedule a repeating task which shows the elapsed times since the task's
             * scheduled execution time.  This demonstrates how a task can inquire about
             * its own scheduled execution time.
             */
            System.out.println( "Scheduling Task 4" );
            task = timer.schedule( new Runnable() {
                public void run() {
                    elapsedTime = System.currentTimeMillis() - task.scheduledExecutionTime();
                    if (elapsedTime >= MAX_TARDINESS) {
                        return;  // Unable to execute task this time
                    }
                    System.out.println( "Task 4 is running.  Elapsed time since this task began: " + elapsedTime );
                }
            }, START_DELAY, REPEAT_DELAY );
            Thread.sleep( START_DELAY + (REPEAT_DELAY * (MAX_TASK_EXECUTIONS + 1/2)) );

            System.out.print( "Cancelling Task 4..." );
            task.cancel();
            System.out.println( "Done!\n" );


            // Cancel the timer
            System.out.println( "Cancelling the timer\n" );
            timer.cancel();
            timer = null;     // Avoid memory leaks

        } catch (Exception e) {
            System.err.println( e.getMessage() );
            return;
        }

    }

    /*
     * Get the Timer service reference.
     */
    public final static String TIMER_CLASS = "com.sun.jes.service.timer.TimerService";
    private TimerService timer = null;  // Reference to the Timer service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (timer == null) {
            if ((sr = bc.getServiceReference( TIMER_CLASS )) != null) {
                timer = (TimerService) bc.getService( sr );
            }
        }
    }

}


