/*
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 *
 * This software is the confidential and proprietary information of Sun
 * Microsystems, Inc. ("Confidential Information"). You shall not
 * disclose such Confidential Information and shall use it only in
 * accordance with the terms of the license agreement you entered into
 * with Sun.
 *
 *
 * TimerTwoTasks
 *
 * An example of how multiple repeating tasks can be scheduled, using
 * each of the four repeating Timer APIs.  The tasks are scheduled to
 * repeat at different times, causing their output to be staggered.
 *
 */

package timertwotasks;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import com.sun.jes.service.timer.TimerService;
import com.sun.jes.service.timer.TimerTask;
import java.util.Date;

public class TimerTwoTasks {

    // Delays (milliseconds)
    private static final long TASK1_DELAY = 0;           // Delay before Task 1 starts
    private static final long TASK1_PERIOD = 100;        // Delay before Task 1 repeats
    private static final long TASK2_DELAY = 0;           // Delay before Task 2 starts
    private static final long TASK2_PERIOD = 300;        // Delay before Task 2 repeats

    // The longest repeat delay for the two tasks
    private static final long MAX_DELAY = Math.max( TASK1_PERIOD, TASK2_PERIOD );

    // The number of times to repeat the set of repeating tasks
    private static final long REPEAT_COUNT = 1;

    // The duration for this thread to sleep until all tasks have completely run
    private static final long END_DELAY = MAX_DELAY * REPEAT_COUNT;

    /*
     * Flags which indicate whether or not a task has fired.
     * These flags must be present here so that run() has
     * access to them.
     */
    private boolean task1Fired = false;
    private boolean task2Fired = false;

    TimerTwoTasks( BundleContext bc ) {

        // Use the bundle context to create the Timer service reference.
        getServices( bc );
        if (timer == null) {
            System.err.println( "Unable to get Timer service reference!" );
            return;
        }


        try {

            // References to the two tasks
            TimerTask task1 = null;
            TimerTask task2 = null;

            // schedule(Runnable runtask, Date firstTime, long period)
            task1Fired = false;     // Task 1 hasn't fired at least once yet
            task2Fired = false;     // Task 2 hasn't fired at least once yet
            System.out.println( "Begin: schedule(Runnable runtask, Date firstTime, long period)" );
            task1 = timer.schedule( new Runnable() {
                public void run() {
                    System.out.println( "Task 1 running" );
                    task1Fired = true;
                }
            } , new Date(), TASK1_PERIOD );
            task2 = timer.schedule( new Runnable() {
                public void run() {
                    System.out.println( "Task 2 running" );
                    task2Fired = true;
                }
            } , new Date(), TASK2_PERIOD );
            Thread.sleep( END_DELAY );
            task1.cancel();
            task2.cancel();
            if (task1Fired == false) {
                System.err.println( "Task 1 didn't fire" );
                return;
            }
            if (task2Fired == false) {
                System.err.println( "Task 2 didn't fire" );
                return;
            }
            System.out.println( "   OK: Tasks fired" );
            System.out.println( "  End: schedule(Runnable runtask, Date firstTime, long period)\n" );


            // schedule(Runnable runtask, long delay, long period)
            task1Fired = false;
            task2Fired = false;
            System.out.println( "Begin: schedule(Runnable runtask, long delay, long period)" );
            task1 = timer.schedule( new Runnable() {
                public void run() {
                    System.out.println( "Task 1 running" );
                    task1Fired = true;
                }
            } , TASK1_DELAY, TASK1_PERIOD );
            task2 = timer.schedule( new Runnable() {
                public void run() {
                    System.out.println( "Task 2 running" );
                    task2Fired = true;
                }
            } , TASK2_DELAY, TASK2_PERIOD );
            Thread.sleep( END_DELAY );
            task1.cancel();
            task2.cancel();
            if (task1Fired == false) {
                System.err.println( "Task 1 didn't fire" );
                return;
            }
            if (task2Fired == false) {
                System.err.println( "Task 2 didn't fire" );
                return;
            }
            System.out.println( "   OK: Tasks fired" );
            System.out.println( "  End: schedule(Runnable runtask, long delay, long period)\n" );


            // scheduleAtFixedRate(Runnable task, Date firstTime, long period)
            task1Fired = false;
            task2Fired = false;
            System.out.println( "Begin: scheduleAtFixedRate(Runnable task, Date firstTime, long period)" );
            task1 = timer.scheduleAtFixedRate( new Runnable() {
                public void run() {
                    System.out.println( "Task 1 running" );
                    task1Fired = true;
                }
            } , new Date(), TASK1_PERIOD );
            task2 = timer.scheduleAtFixedRate( new Runnable() {
                public void run() {
                    System.out.println( "Task 2 running" );
                    task2Fired = true;
                }
            } , new Date(), TASK2_PERIOD );
            Thread.sleep( END_DELAY );
            task1.cancel();
            task2.cancel();
            if (task1Fired == false) {
                System.err.println( "Task 1 didn't fire" );
                return;
            }
            if (task2Fired == false) {
                System.err.println( "Task 2 didn't fire" );
                return;
            }
            System.out.println( "   OK: Tasks fired" );
            System.out.println( "  End: scheduleAtFixedRate(Runnable task, Date firstTime, long period)\n" );


            // scheduleAtFixedRate(Runnable task, long delay, long period)
            task1Fired = false;
            task2Fired = false;
            System.out.println( " Begin: scheduleAtFixedRate(Runnable task, long delay, long period)" );
            task1 = timer.scheduleAtFixedRate( new Runnable() {
                public void run() {
                    System.out.println( "Task 1 running" );
                    task1Fired = true;
                }
            } , TASK1_DELAY, TASK1_PERIOD );
            task2 = timer.scheduleAtFixedRate( new Runnable() {
                public void run() {
                    System.out.println( "Task 2 running" );
                    task2Fired = true;
                }
            } , TASK2_DELAY, TASK2_PERIOD );
            Thread.sleep( END_DELAY );
            task1.cancel();
            task2.cancel();
            if (task1Fired == false) {
                System.err.println( "Task 1 didn't fire" );
                return;
            }
            if (task2Fired == false) {
                System.err.println( "Task 2 didn't fire" );
                return;
            }
            System.out.println( "   OK: Tasks fired" );
            System.out.println( "  End: scheduleAtFixedRate(Runnable task, long delay, long period)\n" );

        } catch (Exception e) {
            System.err.println( e.getMessage() );
        }


        // Test the cancel() method and also use it to cancel any runaway tasks
        // and to stop the timer (scheduler) service itself.
        System.out.println( "Cancelling timer" );
        timer.cancel();
        timer = null;  // Avoid memory leaks

    }

    /*
     * Get the Timer service reference.
     */
    public final static String TIMER_CLASS = "com.sun.jes.service.timer.TimerService";
    private TimerService timer = null;  // Reference to the Timer service object
    synchronized void getServices( BundleContext bc ) {
        ServiceReference sr = null;

        if (timer == null) {
            if ((sr = bc.getServiceReference( TIMER_CLASS )) != null) {
                timer = (TimerService) bc.getService( sr );
            }
        }
    }

}
